import os, configparser, errno


def get_config():
    user_config_dir = os.path.expanduser("~")
    config = configparser.ConfigParser(allow_no_value=True)

    filename = user_config_dir + "/.config/spotui/.spotuirc"
    if not os.path.exists(os.path.dirname(filename)):
        try:
            os.makedirs(os.path.dirname(filename))
        except OSError as exc:  # prevent race condition
            if exc.errno != errno.EEXIST:
                raise

    # if config file doesn't exist
    if not os.path.isfile(filename):
        _create_default_config(config, filename)

    config.read(filename)

    # check config is valid
    _check_config(config, filename)

    return config


def _create_default_config(configparser, filename):
    configparser.add_section("spotify_api")
    configparser.set("spotify_api", "user_name", "*your spotify username*")
    configparser.set(
        "spotify_api",
        "; register an application here: https://developer.spotify.com/dashboard/login",
    )
    configparser.set("spotify_api", "client_id", "*your application ID*")
    configparser.set("spotify_api", "client_secret",
                     "*your application secret*")
    configparser.set("spotify_api", "redirect_uri",
                     "http://localhost:8888/auth")

    configparser.add_section("other")
    configparser.set("other", "use_nerd_fonts", "yes")
    configparser.set("other", "config_version", "1")

    configparser.write(open(filename, "w"))


def _check_config(configparser, filename):
    user_name = configparser.get("spotify_api", "user_name")
    client_id = configparser.get("spotify_api", "client_id")
    client_secret = configparser.get("spotify_api", "client_secret")

    if (client_id == "*your application ID*"
            or client_secret == "*your application secret*"
            or user_name == "*your spotify username*"):
        print("You need to configure: " + filename)
        exit(1)



import curses
from spotui.src.util import truncate
from spotui.src.menu import Menu
from spotui.src.component import Component


class DeviceMenu(Component):
    def __init__(self, stdscr, api, select_device, close):
        self.stdscr = stdscr
        self.api = api
        self.select_device = select_device
        self.close = close
        self.active = True
        self.popup = True
        self.title = "Select a Device"
        self.interactive = True
        self.items = api.get_devices()
        self.restart()

    def restart(self):
        self.items = self.api.get_devices()
        scry, scrx = self.stdscr.getmaxyx()
        box_height = round(scry / 2)
        box_width = round(scrx / 2.5)
        self.startx = round((scrx / 2) - (box_width / 2))
        self.endx = self.startx + box_width
        self.starty = round((scry / 2) - (box_height / 2))
        self.endy = self.starty + box_height
        self.component = Menu(
            self.stdscr,
            list(map(self.__map_devices, self.items))
            if self.items and len(self.items) > 0 else [],
            self.starty,
            self.startx,
            self.endy,
            self.endx,
        )

    def __map_devices(self, item):
        available_space = self.endx - self.startx - 6
        item["text"] = truncate(item["text"], available_space)

        def handler():
            if item and "id" in item:
                self.select_device(item["id"])

        item["handler"] = handler
        return item

    def receive_input(self, key):
        if ((key == curses.KEY_ENTER or key in [10, 13]) and self.items
                and len(self.items) > 0):
            self.items[self.component.selected]["handler"]()
            self.close()
        else:
            self.component.receive_input(key)
